package com.appier.appier_flutter

import android.app.Activity
import android.content.Context
import android.util.Log
import androidx.annotation.NonNull
import com.quantumgraph.sdk.QG
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import org.json.JSONObject

/** AppierFlutterPlugin */
class AppierFlutterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  private lateinit var context: Context
  private lateinit var activity: Activity

  private val TAG = "APPIER"

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "appier_flutter")
    context = flutterPluginBinding.applicationContext
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
      "configure" -> {
        val appId = call.argument<String>("appId")
        // senderId key is always present since controlled by Appier Plugin
        call.argument<String>("senderId")?.also { senderId ->
          QG.initializeSdk(activity.application, appId, senderId)
        } ?: run {
          QG.initializeSdk(activity.application, appId)
        }

        QG.initializeInApp(activity.application)
        QG.setInAppCurrActivity(activity)

        result.success(null)
      }

      "setUniversalLinkDomains" -> {
        Log.d(TAG, "${call.method}: is not supported in android")
        result.success(null)
      }

      "logEvent" -> {
        call.argument<String>("eventName")?.also { eventName ->
          val params = call.argument<HashMap<String, Any?>>("parameters")
                  ?: HashMap()
          val eventInfo = JSONObject(params.toString())

          val vts = call.argument<Double>("vts") ?: 0.0

          call.argument<String>("vtsCurr")?.also { vtsCurr ->
            QG.getInstance(context).logEvent(eventName, eventInfo, vts, vtsCurr)
          } ?: run {
            QG.getInstance(context).logEvent(eventName, eventInfo, vts)
          }
        } ?: run {
          Log.d(TAG, "${call.method}: invalid eventName passed")
        }
        result.success(null)
      }

      "flush" -> {
        QG.getInstance(context).flush()
        result.success(null)
      }

      "setUserId" -> {
        call.argument<String>("userId")?.also {
          QG.getInstance(context).setUserId(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setName" -> {
        call.argument<String>("name")?.also {
          QG.getInstance(context).setName(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setFirstName" -> {
        call.argument<String>("firstName")?.also {
          QG.getInstance(context).setFirstName(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setLastName" -> {
        call.argument<String>("lastName")?.also {
          QG.getInstance(context).setLastName(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setCity" -> {
        call.argument<String>("city")?.also {
          QG.getInstance(context).setCity(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setEmail" -> {
        call.argument<String>("email")?.also {
          QG.getInstance(context).setEmail(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setPhoneNumber" -> {
        call.argument<String>("phoneNo")?.also {
          QG.getInstance(context).setPhoneNumber(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setDayOfBirth" -> {
        call.argument<Int>("day")?.also {
          QG.getInstance(context).setDayOfBirth(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setMonthOfBirth" -> {
        call.argument<Int>("month")?.also {
          QG.getInstance(context).setMonthOfBirth(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setYearOfBirth" -> {
        call.argument<Int>("year")?.also {
          QG.getInstance(context).setYearOfBirth(it)
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setCustomKey" -> {
        call.argument<String>("key")?.also { key ->
          call.argument<Any>("value")?.also { value ->
            QG.getInstance(context).setCustomUserParameter(key, value)
          } ?: run {
            QG.getInstance(context).setCustomUserParameter(key, null)
          }
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setAttributionWindow" -> {
        call.argument<Int>("seconds")?.also {
          QG.getInstance(context).setAttributionWindow(it.toLong())
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      "setClickAttributionWindow" -> {
        call.argument<Int>("seconds")?.also {
          QG.getInstance(context).setClickAttributionWindow(it.toLong())
        } ?: run {
          Log.d(TAG, "${call.method}: Invalid parameter passed")
        }
        result.success(null)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {
    // TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onDetachedFromActivityForConfigChanges() {
    // TODO("Not yet implemented")
  }
}
