import 'dart:async';

import 'package:flutter/services.dart';

class AppierFlutter {
  static const MethodChannel _channel = const MethodChannel('appier_flutter');

  /// Configures the SDK
  ///
  /// ```
  /// AppierFlutter.configure("APP_ID", senderId: "SENDER_ID",
  /// appGroup: "APP_GROUP", isDev: true);
  /// ```
  /// [senderId] is used in Andriod
  /// [appGroup] is used in iOS. It will help to share data between app target
  /// and extensions. App Group is 'Required' to log events from extension and
  /// also to track certain events from the rich push notification.
  static Future<void> configure(String appId,
      {String senderId, String appGroup, bool isDev = false}) async {
    if (appId == null || appId.trim().isEmpty) {
      print("APPIER: Invalid AppId passed");
      return;
    }

    isConfigured = true;

    await _channel.invokeMethod('configure', {
      'appId': appId,
      'senderId': senderId,
      'appGroup': appGroup,
      'isDev': isDev
    });
  }

  /// Set the Associated Domains for Universal Links
  ///
  /// ```
  /// AppierFlutter.setUniversalLinkDomains(["www.domain.com"])
  /// ```
  /// This is used in iOS
  static Future<void> setUniversalLinkDomains(List<String> linkDomains) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel
        .invokeMethod('setUniversalLinkDomains', {'linkDomains': linkDomains});
  }

  /// Sends any event in your app to the Appier server
  ///
  /// ```
  /// AppierFlutter.logEvent("product_purchased", parameters: { "param1": 2,
  /// "param2": null, "param3": "appier"}, vts: 100, vtsCurr: "USD")
  /// ```
  /// [vts] and [vtsCurr] are optional.
  /// [vts] - valueToSum. It is a monetary value(number) associated to the event
  /// [vtsCurr] - currency code of the value to sum, for example: "USD" or "NTD"
  static Future<void> logEvent(String eventName,
      {Map<String, dynamic> parameters, double vts, String vtsCurr}) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }

    if (eventName == null || eventName.trim().isEmpty) {
      print("APPIER: Invalid Event Name passed");
      return;
    }

    await _channel.invokeMethod('logEvent', {
      'eventName': eventName,
      'parameters': parameters,
      'vts': vts,
      'vtsCurr': vtsCurr
    });
  }

  /// Immediately uploads queued data to the Appier server
  ///
  /// ```
  /// AppierFlutter.flush()
  /// ```
  /// By default, queued data is flushed to the Appier servers every 15 seconds
  /// (the default for flushInterval). You only need to call this method
  /// manually if you want to force a flush at a particular moment.
  static Future<void> flush() async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('flush', {});
  }

  /// Set the unique user id for your users
  ///
  /// ```
  /// AppierFlutter.setUserId("2031203056")
  /// ```
  static Future<void> setUserId(String userId) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setUserId', {'userId': userId});
  }

  /// Set the name of the user
  ///
  /// ```
  /// AppierFlutter.setName("Gary.Wu")
  /// ```
  static Future<void> setName(String name) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setName', {'name': name});
  }

  /// Set the firstName of the user
  ///
  /// ```
  /// AppierFlutter.setFirstName("Gary")
  /// ```
  static Future<void> setFirstName(String firstName) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setFirstName', {'firstName': firstName});
  }

  /// Set the lastName of the user
  ///
  /// ```
  /// AppierFlutter.setLastName("Wu")
  /// ````
  static Future<void> setLastName(String lastName) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setLastName', {'lastName': lastName});
  }

  /// Set the city of the user
  ///
  /// ```
  /// AppierFlutter.setCity("Taipei")
  /// ```
  static Future<void> setCity(String city) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setCity', {'city': city});
  }

  /// Set the email of the user
  ///
  /// ```
  /// AppierFlutter.setEmail("feedback@appier.com")
  /// ```
  static Future<void> setEmail(String email) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setEmail', {'email': email});
  }

  /// Set the phoneNumber of the user
  ///
  /// ```
  /// AppierFlutter.setPhoneNumber("02-8865252")
  /// ```
  static Future<void> setPhoneNumber(String phoneNo) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setPhoneNumber', {'phoneNo': phoneNo});
  }

  /// Set the day of DOB of the user
  ///
  /// ```
  /// AppierFlutter.setDayOfBirth(29)
  /// ```
  static Future<void> setDayOfBirth(int day) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setDayOfBirth', {'day': day});
  }

  /// Set the month of DOB of the user
  ///
  /// ```
  /// AppierFlutter.setMonthOfBirth(7)
  /// ````
  static Future<void> setMonthOfBirth(int month) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setMonthOfBirth', {'month': month});
  }

  /// Set the year of DOB of the user
  ///
  /// ```
  /// AppierFlutter.setYearOfBirth(1992)
  /// ```
  static Future<void> setYearOfBirth(int year) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setYearOfBirth', {'year': year});
  }

  /// Set any custom key for your user
  ///
  /// ````
  /// AppierFlutter.setCustomKey("rating", 3.5)
  /// ```
  static Future<void> setCustomKey(String key, dynamic value) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }

    if (key == null || key.trim().isEmpty) {
      print("APPIER: Invalid key passed");
      return;
    }

    if (value is String ||
        value is double ||
        value is int ||
        value is num ||
        value is bool) {
      await _channel.invokeMethod('setCustomKey', {'key': key, 'value': value});
    } else {
      print(
          "APPIER: value must be string, number(num or int or double) or boolean");
    }
  }

  /// Sets the View Through Attribution Window for event attribution
  ///
  /// ```
  /// AppierFlutter.setAttributionWindow(10000)
  /// ```
  /// Pass [seconds] as 0 to disable view-through attribution
  static Future<void> setAttributionWindow(int seconds) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel.invokeMethod('setAttributionWindow', {'seconds': seconds});
  }

  /// Sets the Click Through Attribution Window for event attribution
  ///
  /// ```
  /// AppierFlutter.setClickAttributionWindow(20000)
  /// ```
  /// Pass [seconds] as 0 to disable click attribution
  static Future<void> setClickAttributionWindow(int seconds) async {
    if (!isConfigured) {
      print("APPIER: Must configure the SDK first!");
      return;
    }
    await _channel
        .invokeMethod('setClickAttributionWindow', {'seconds': seconds});
  }
}

bool isConfigured = false;
